@@ -1,29 +1,67 @@
+# coding: utf-8
+
 """
-A library for Stack Overflow question notifications.
+A library and command line tool to notify you of relevant
+questions as soon as they're posted on Stack Overflow.
 """
 __author__ = 'Matt Deacalion Stevens'
-__version__ = '0.2'
+__version__ = '0.3.0'
 
+import re
 import time
-import urllib.parse
 import json
+import inspect
+import importlib
 import requests
 
+# Python 3
+try:
+    from urllib import parse as urllib
+# Python 2
+except ImportError:
+    import urllib as urllib
 
-class QuestionMetaClass(type):
+
+class RulesMixin(object):
     """
-    Find all the methods within the `Question` class beginning with `is_` and
-    adds them to the `rules` attribute. This is so all of the `rules` can be
-    dynamically run at once.
+    A mixin that can provides validation 'rules' to a Question. These rules are
+    properties and if they all return True the question is considered valid.
     """
-    def __new__(cls, name, bases, attrs):
-        attrs['rules'] = {
-            k: v for k, v in attrs.items() if k.startswith('is_')
-        }
-        return super().__new__(cls, name, bases, attrs)
+    @property
+    def is_not_answered(self):
+        """
+        Return True if this question has no answers.
+        """
+        return not self.answered
+
+    @property
+    def is_new(self):
+        """
+        Return True if the question is less than 6 hours old.
+        """
+        return self.creation_date > time.time() - (60 * 60 * 6)
+
+    @property
+    def is_including_positive_tags(self):
+        """
+        Question is tagged with at least one of our `positive` tags.
+        """
+        return any(tag in self.tags for tag in self.get_positive_tags())
+
+    @property
+    def is_excluding_negative_tags(self):
+        """
+        Question does not contain a single `negative` tag.
+        """
+        negative_tags = self.get_negative_tags()
+
+        if not negative_tags:
+            return True
+
+        return not any(tag in self.tags for tag in negative_tags)
 
 
-class Question(metaclass=QuestionMetaClass):
+class Question(object):
     """
     An entity to contain a single question from any Stack Exchange web site.
     Also has methods of validating and classifying itself.
@@ -55,6 +93,12 @@ class Question(metaclass=QuestionMetaClass):
         for attribute in question_attributes:
             setattr(self, attribute, kwargs.get(attribute))
 
+    def __str__(self):
+        """
+        Overriden to display a short URL and short title.
+        """
+        return '{} {}'.format(self.get_short_link(), self.get_short_title(70))
+
     def __getitem__(self, attribute):
         """
         Allow attributes to be accessible in the same way as a dict.
@@ -93,37 +137,33 @@ class Question(metaclass=QuestionMetaClass):
         return all([getattr(self, prop) for prop in self.rules])
 
     @property
-    def is_not_answered(self):
+    def rules(self):
         """
-        Return True if this question has no answers.
+        Find all the methods within the `Question` class beginning with `is_`
+        and return a list of their names.
         """
-        return not self.answered
+        methods = dir(self.__class__)
+        return [rule for rule in methods if rule.startswith('is_')]
 
-    @property
-    def is_new(self):
+    def get_short_link(self):
         """
-        Return True if the question is less than 6 hours old.
+        Returns the StackExchange URL without the text.
         """
-        return self.creation_date > time.time() - (60 * 60 * 6)
+        short_link = re.match('http://([^/]+/){2}\d+/', self.link)
 
-    @property
-    def is_including_positive_tags(self):
-        """
-        Question is tagged with at least one of our `positive` tags.
-        """
-        return any(tag in self.tags for tag in self.get_positive_tags())
+        if short_link:
+            return short_link.group()
 
-    @property
-    def is_excluding_negative_tags(self):
-        """
-        Question does not contain a single `negative` tag.
-        """
-        negative_tags = self.get_negative_tags()
+        return self.link
 
-        if not negative_tags:
-            return True
+    def get_short_title(self, length=50):
+        """
+        Returns the question title truncated to `length`.
+        """
+        if len(self.title) > length:
+            return self.title[:length] + '...'
 
-        return not any(tag in self.tags for tag in negative_tags)
+        return self.title
 
     def get_positive_tags(self):
         """
@@ -140,7 +180,7 @@ class Question(metaclass=QuestionMetaClass):
         return self.negative_tags
 
 
-class Retriever:
+class Retriever(object):
     """
     Responsible for retrieving the questions from a Stack Exchange web site
     and creating `Question` instances with the results.
@@ -167,6 +207,7 @@ class Retriever:
         self.question_cls = question_cls
         self.bookmark = bookmark
         self._active = True
+        self._throttle_wait_time = 120
 
     @property
     def feed_url(self):
@@ -175,7 +216,7 @@ class Retriever:
         """
         return '?'.join([
             self.get_base_url(),
-            urllib.parse.urlencode(self.params),
+            urllib.urlencode(self.params),
         ])
 
     @property
@@ -185,6 +226,14 @@ class Retriever:
         """
         return self.last_retrieved > time.time() - self.interval
 
+    def throttled(self):
+        """
+        This is a `hook` method that is run when our feed requests to Stack
+        Exchange are throttled. Override this in subclasses to do something
+        that can work around the throttle in a nice way.
+        """
+        raise NotImplementedError
+
     def get_base_params(self):
         """
         Returns a list of the default parameters to use in the API URL.
@@ -207,8 +256,19 @@ class Retriever:
         """
         Retrieves the question feed and saves the time it happened.
         """
+        response = requests.get(self.get_base_url(), params=self.params)
+
+        while not response:
+            try:
+                self.throttled()
+            except NotImplementedError:
+                print('throttled() method not implemented. Waiting instead.')
+                time.sleep(self._throttle_wait_time)
+
+            response = requests.get(self.get_base_url(), params=self.params)
+
         self.last_retrieved = time.time()
-        return requests.get(self.get_base_url(), params=self.params)
+        return response
 
     def _parse_response(self, response):
         """
@@ -271,8 +331,81 @@ class Retriever:
                 yield new_question
 
 
-class Notifier:
+class Notifier(object):
     """
     Contains methods of notifying the user about certain types of question.
     """
     pass
+
+
+class ClassLoader(object):
+    """
+    Contains helper methods to dynamically load user specified Python classes.
+    """
+    def split_import_path(self, import_path):
+        """
+        Takes a Python import path as a string and returns a tuple containing
+        the module and the identifier.
+        """
+        components = import_path.split('.')
+        identifier = components.pop()
+        return ('.'.join(components), identifier)
+
+    def load(self, import_path):
+        """
+        Takes a Python import path, does a few checks on the class, then
+        imports and returns it.
+        """
+        module_path, identifier_path = self.split_import_path(import_path)
+
+        try:
+            mod = importlib.import_module(module_path)
+        except ImportError:
+            msg = 'Could not import `{}`, is it on sys.path?'
+            raise ImportError(msg.format(module_path))
+
+        cls = getattr(mod, identifier_path)
+
+        if not inspect.isclass(cls):
+            raise ImportError('`{}` must be a class'.format(identifier_path))
+
+        return cls
+
+    def load_question_class(self, import_path):
+        """
+        Takes a Python import string, then loads and returns the Python class
+        if it's a subclass of the `stack_watcher.Question` class.
+        """
+        cls = self.load(import_path)
+
+        if not issubclass(cls, Question):
+            msg = '`{}` must be a subclass of `stack_watcher.Question`.'
+            raise ImportError(msg.format(import_path))
+
+        return cls
+
+    def load_retriever_class(self, import_path):
+        """
+        Takes a Python import string, then loads and returns the Python class
+        if it's a subclass of the `stack_watcher.Retriever` class.
+        """
+        cls = self.load(import_path)
+
+        if not issubclass(cls, Retriever):
+            msg = '`{}` must be a subclass of `stack_watcher.Retriever`.'
+            raise ImportError(msg.format(import_path))
+
+        return cls
+
+    def load_notifier_class(self, import_path):
+        """
+        Takes a Python import string, then loads and returns the Python class
+        if it's a subclass of the `stack_watcher.Notifier` class.
+        """
+        cls = self.load(import_path)
+
+        if not issubclass(cls, Notifier):
+            msg = '`{}` must be a subclass of `stack_watcher.Notifier`.'
+            raise ImportError(msg.format(import_path))
+
+        return cls
