@@ -1,10 +1,23 @@
+# coding: utf-8
+
 import time
 import unittest
-import urllib.parse
-from unittest import mock
-from pathlib import Path
-from stack_watcher import Retriever, Question, Notifier
+from stack_watcher import (
+    Retriever, Question, Notifier, RulesMixin, ClassLoader
+)
 
+# Python 3
+try:
+    from urllib.parse import parse_qs
+    from urllib import parse as urllib
+    from unittest import mock
+    from pathlib import Path
+# Python 2
+except ImportError:
+    import urllib as urllib
+    from urlparse import parse_qs
+    import mock
+    from unipath import Path
 
 def read_file(path):
     path = str(Path(__file__, '..', 'resources', path).resolve())
@@ -13,6 +26,34 @@ def read_file(path):
         return fp.read().strip()
 
 
+class Quest(RulesMixin, Question):
+    """
+    A question with rules, which it can use to determine whether it's valid.
+    """
+    pass
+
+
+class SubRetriever(Retriever):
+    """
+    Only exists for unit testing the `ClassLoader` class.
+    """
+    pass
+
+
+class SubNotifier(Notifier):
+    """
+    Only exists for unit testing the `ClassLoader` class.
+    """
+    pass
+
+
+class LoadableClass(object):
+    """
+    Only exists to for unit testing the `ClassLoader` class.
+    """
+    loaded = True
+
+
 class RetrieverTest(unittest.TestCase):
 
     def test_retriever_class_exists(self):
@@ -26,8 +67,8 @@ class RetrieverTest(unittest.TestCase):
         Does the `feed_url` property exist and work correctly?
         """
         retriever = Retriever()
-        base_url, query = urllib.parse.splitquery(retriever.feed_url)
-        params = urllib.parse.parse_qs(query)
+        base_url, query = urllib.splitquery(retriever.feed_url)
+        params = parse_qs(query)
 
         self.assertTrue(hasattr(retriever, 'feed_url'))
 
@@ -38,8 +79,8 @@ class RetrieverTest(unittest.TestCase):
         self.assertEqual('stackoverflow', params.get('site')[0])
 
         retriever = Retriever(tag='professionalism', site='workplace')
-        base_url, query = urllib.parse.splitquery(retriever.feed_url)
-        params = urllib.parse.parse_qs(query)
+        base_url, query = urllib.splitquery(retriever.feed_url)
+        params = parse_qs(query)
 
         # check custom parameters
         self.assertEqual('professionalism', params.get('tagged')[0])
@@ -260,6 +301,53 @@ class RetrieverTest(unittest.TestCase):
 
         self.assertEqual(mock_sleep.call_count, r.interval)
 
+    @mock.patch('stack_watcher.requests.get', autospec=True)
+    def test_get_feed_no_throttling(self, mock_requests):
+        """
+        Does `_get_feed` return ok if there is no throttling?
+        """
+        mock_requests.return_value = True
+        Retriever()._get_feed()
+
+        self.assertEqual(mock_requests.called, 1)
+
+    @mock.patch('stack_watcher.requests.get', autospec=True)
+    @mock.patch('stack_watcher.time.sleep', autospec=True)
+    def test_get_feed_throttled_not_implemented(
+        self,
+        mock_sleep,
+        mock_requests,
+    ):
+        """
+        Does `_get_feed` sleep when we're throttled and there is no
+        `throttled` method implemented?
+        """
+        mock_requests.side_effect = iter([False, True])
+        Retriever()._get_feed()
+
+        self.assertEqual(mock_requests.call_count, 2)
+        self.assertTrue(mock_sleep.called)
+
+    @mock.patch('stack_watcher.requests.get', autospec=True)
+    @mock.patch('stack_watcher.time.sleep', autospec=True)
+    @mock.patch.object(Retriever, 'throttled', autospec=True)
+    def test_get_feed_throttled(
+        self,
+        mock_throttled,
+        mock_sleep,
+        mock_requests,
+    ):
+        """
+        Does `_get_feed` call the `throttled` method with no delay if it's
+        implemented ok?
+        """
+        mock_requests.side_effect = iter([False, True])
+        Retriever()._get_feed()
+
+        self.assertEqual(mock_requests.call_count, 2)
+        self.assertFalse(mock_sleep.called)
+        self.assertTrue(mock_throttled.called)
+
 
 class QuestionTest(unittest.TestCase):
 
@@ -290,7 +378,7 @@ class QuestionTest(unittest.TestCase):
         """
         Utility function to create a Question that adheres to all rules.
         """
-        question = Question.from_dict(dict(**self.question))
+        question = Quest.from_dict(dict(**self.question))
         question.positive_tags = self.question['tags']
 
         return question
@@ -452,12 +540,12 @@ class QuestionTest(unittest.TestCase):
         Does the `is_not_answered` rule exist and work correctly?
         """
         self.question['is_answered'] = True
-        answered_question = Question.from_dict(dict(**self.question))
+        answered_question = Quest.from_dict(dict(**self.question))
 
         self.question['is_answered'] = False
-        unanswered_question = Question.from_dict(dict(**self.question))
+        unanswered_question = Quest.from_dict(dict(**self.question))
 
-        self.assertTrue(hasattr(Question(), 'is_not_answered'))
+        self.assertTrue(hasattr(Quest(), 'is_not_answered'))
         self.assertTrue(unanswered_question.is_not_answered)
         self.assertFalse(answered_question.is_not_answered)
 
@@ -468,7 +556,7 @@ class QuestionTest(unittest.TestCase):
         new_question = self.create_valid_question()
 
         self.question['creation_date'] = int(time.time()) - (60 * 60 * 6)
-        old_question = Question.from_dict(dict(**self.question))
+        old_question = Quest.from_dict(dict(**self.question))
 
         self.assertTrue(new_question.is_new)
         self.assertFalse(old_question.is_new)
@@ -480,7 +568,7 @@ class QuestionTest(unittest.TestCase):
         includes_positive_tags = self.create_valid_question()
 
         self.question['tags'] = []
-        excludes_positive_tags = Question.from_dict(dict(**self.question))
+        excludes_positive_tags = Quest.from_dict(dict(**self.question))
 
         self.assertTrue(includes_positive_tags.is_including_positive_tags)
         self.assertFalse(excludes_positive_tags.is_including_positive_tags)
@@ -492,12 +580,93 @@ class QuestionTest(unittest.TestCase):
         excludes_negative_tags = self.create_valid_question()
 
         self.question['tags'] = ['Russell Crowe', 'Golf', 'Bono']
-        includes_negative_tags = Question.from_dict(dict(**self.question))
+        includes_negative_tags = Quest.from_dict(dict(**self.question))
         includes_negative_tags.negative_tags = self.question['tags']
 
         self.assertTrue(excludes_negative_tags.is_excluding_negative_tags)
         self.assertFalse(includes_negative_tags.is_excluding_negative_tags)
 
+    def test_subclass_with_mixin_has_rules(self):
+        """
+        Do `Question` subclasses with rules mixed in have the `rules`
+        attribute?
+        """
+        self.assertEqual(len(Quest().rules), 4)
+
+    def test_get_short_link(self):
+        """
+        Does the `get_short_link` method work correctly?
+        """
+        pre = 'http://stackoverflow.com/questions/'
+
+        urls = [
+            (
+                pre + '1327369/extract-contents-of-regex',
+                pre + '1327369/',
+            ),
+            (
+                pre + '23606124/python-django-bidi-brackets-issue-in-html-sel',
+                pre + '23606124/',
+            ),
+            (
+                pre + '24197506/how-to-get-1-100-between-numbers-in-python',
+                pre + '24197506/',
+            ),
+            (
+                pre + '24196040/retrieving-array-values-in-autohotkey-script',
+                pre + '24196040/',
+            ),
+            (
+                pre + '24197437/geektool-not-working-with-python3',
+                pre + '24197437/',
+            ),
+            (
+                pre + '4/when-setting-a-forms-opacity-should-i-use-a-decimal',
+                pre + '4/',
+            ),
+            (
+                'http://dirtymonkey.co.uk',
+                'http://dirtymonkey.co.uk',
+            ),
+            (
+                '',
+                '',
+            ),
+        ]
+
+        for url in urls:
+            self.assertEqual(Quest(link=url[0]).get_short_link(), url[1])
+
+    def test_get_short_title(self):
+        """
+        Does the `get_short_title` method work correctly?
+        """
+        titles = [
+            ('', ''),
+            ('Matt', 'Matt'),
+            ('Matt Deacalion Stevens', 'Matt ...'),
+            ('On this Day I Complete my Thirty-Sixth Year', 'On th...'),
+            ('And Did Those Feet In Ancient Time', 'And D...'),
+            ('Auguries of Innocence', 'Augur...'),
+        ]
+
+        for title in titles:
+            self.assertEqual(
+                Quest(title=title[0]).get_short_title(length=5),
+                title[1],
+            )
+
+    def test_question_string_representation(self):
+        """
+        Does `Question` implement `__str__` so it shows a short URL and title?
+        """
+        question = Quest(
+            link='http://stackoverflow.com/questions/1327369/extract-contents',
+            title='A really long title... ' * 10,
+        )
+
+        self.assertEqual(len(str(question)), 117)
+
 
 class NotifierTest(unittest.TestCase):
 
@@ -506,3 +675,141 @@ class NotifierTest(unittest.TestCase):
         Does the `Notifier` class exist?
         """
         self.assertTrue('Notifier' in globals())
+
+
+class ClassLoaderTest(unittest.TestCase):
+
+    def test_class_loader_class_exists(self):
+        """
+        Does the `ClassLoader` class exist?
+        """
+        self.assertTrue('ClassLoader' in globals())
+
+    def test_split_module_and_identifier(self):
+        """
+        Does the `split_import_path` method correctly divide a Python import
+        path into the module and identifier components?
+        """
+        loader = ClassLoader()
+
+        self.assertEqual(
+            loader.split_import_path('stark'),
+            ('', 'stark'),
+        )
+        self.assertEqual(
+            loader.split_import_path('stark.lannister'),
+            ('stark', 'lannister'),
+        )
+        self.assertEqual(
+            loader.split_import_path('stark.lannister.baratheon'),
+            ('stark.lannister', 'baratheon'),
+        )
+
+    def test_load_path_exception_wrong_path(self):
+        """
+        Does the `load` method raise an `ImportError` if the import fails?
+        """
+        loader = ClassLoader()
+
+        with self.assertRaises(ImportError) as import_exception:
+            loader.load('Lord.Baelish')
+
+        self.assertEqual(
+            str(import_exception.exception),
+            'Could not import `Lord`, is it on sys.path?',
+        )
+
+    def test_load_path_exception_not_class(self):
+        """
+        Does the `load` method raise an `ImportError` if the identifier is not
+        a class?
+        """
+        loader = ClassLoader()
+
+        with self.assertRaises(ImportError) as import_exception:
+            loader.load('re.IGNORECASE')
+
+        self.assertEqual(
+            str(import_exception.exception),
+            '`IGNORECASE` must be a class',
+        )
+
+    def test_load_path_successfully_loads_class(self):
+        """
+        Does the `load` method return a Python class if everything is ok?
+        """
+        loader = ClassLoader()
+        cls = loader.load('stack_watcher.test.test_main.LoadableClass')
+        self.assertEqual(cls().loaded, True)
+
+    def test_load_question_class_subclass_exception(self):
+        """
+        Does the `load_question_class` raise an exception if the identifier is
+        not a subclass of `stack_watcher.Question`?
+        """
+        loader = ClassLoader()
+
+        with self.assertRaises(ImportError) as import_exception:
+            loader.load_question_class('io.StringIO')
+
+        self.assertEqual(
+            str(import_exception.exception),
+            '`io.StringIO` must be a subclass of `stack_watcher.Question`.',
+        )
+
+    def test_load_question_class(self):
+        """
+        Does the `load_question_class` return a subclass of `Question` if
+        everything is ok?
+        """
+        loader = ClassLoader()
+        cls = loader.load('stack_watcher.test.test_main.Quest')
+        self.assertEqual(cls.__name__, 'Quest')
+
+    def test_load_retriever_class_subclass_exception(self):
+        """
+        Does the `load_retriever_class` raise an exception if the identifier is
+        not a subclass of `stack_watcher.Retriever`?
+        """
+        loader = ClassLoader()
+
+        with self.assertRaises(ImportError) as import_exception:
+            loader.load_retriever_class('io.StringIO')
+
+        self.assertEqual(
+            str(import_exception.exception),
+            '`io.StringIO` must be a subclass of `stack_watcher.Retriever`.',
+        )
+
+    def test_load_retriever_class(self):
+        """
+        Does the `load_retriever_class` return a subclass of `Retriever` if
+        everything is ok?
+        """
+        loader = ClassLoader()
+        cls = loader.load('stack_watcher.test.test_main.SubRetriever')
+        self.assertEqual(cls.__name__, 'SubRetriever')
+
+    def test_load_notifier_class_subclass_exception(self):
+        """
+        Does the `load_notifier_class` raise an exception if the identifier is
+        not a subclass of `stack_watcher.Notifier`?
+        """
+        loader = ClassLoader()
+
+        with self.assertRaises(ImportError) as import_exception:
+            loader.load_notifier_class('io.StringIO')
+
+        self.assertEqual(
+            str(import_exception.exception),
+            '`io.StringIO` must be a subclass of `stack_watcher.Notifier`.',
+        )
+
+    def test_load_notifier_class(self):
+        """
+        Does the `load_notifier_class` return a subclass of `Notifier` if
+        everything is ok?
+        """
+        loader = ClassLoader()
+        cls = loader.load('stack_watcher.test.test_main.SubNotifier')
+        self.assertEqual(cls.__name__, 'SubNotifier')
